from procgraph import Generator, Block


class FixFrameRate(Generator):
    Block.alias("fix_frame_rate")
    Block.config("fps")

    Block.input("x")
    Block.output("xreg")

    def init(self):
        self.first_timestamp = None
        self.dt = 1.0 / self.config.fps
        self.nframes_rec = 0
        self.nframes_sent = 0
        self.current_time = None

    def update(self):

        # self.info('updated: T = %s' % self.get_input_timestamp(0))
        if self.first_timestamp is None:
            self.first_timestamp = self.get_input_timestamp(0)
            if self.first_timestamp is None:
                self.info("Strange, the input timestamp is none.")
                return
            self.next_frame_to_output = self.first_timestamp - self.dt
            self.prev_x = self.get_input(0)

        x = self.get_input(0)

        x_updated = self.get_input_timestamp(0) != self.current_time
        if x_updated:
            self.nframes_rec += 1
            self.next_x = x

        self.current_time = self.get_input_timestamp(0)

        msg = "x: %.3f  next: %.3f" % (self.current_time, self.next_frame_to_output)

        msg += "   (f in: %5d out: %5d)" % (self.nframes_rec, self.nframes_sent)

        if self.current_time > self.next_frame_to_output:
            self.set_output("xreg", self.prev_x, self.next_frame_to_output)
            self.next_frame_to_output += self.dt
            self.nframes_sent += 1
            msg += " -> writing"
        else:

            msg += ""

        # self.info(msg)

    def next_data_status(self):
        if self.first_timestamp is None:
            res = (False, None)  # changed from True
        else:
            if self.current_time > self.next_frame_to_output:
                res = (True, self.next_frame_to_output)
            else:
                res = (False, None)
                self.prev_x = self.next_x

        return res



from procgraph.core.registrar_other import simple_block

# import numpy as np
# from . import cv as cv2
# from procgraph_cv.conversions import numpy_to_cv
# from reprep.graphics.zoom import rgb_zoom
from contracts import contract
from procgraph_images.filters import torgb


@simple_block
@contract(gray="array[HxW]")
def simple_detector_demo(gray):
    import cv2

    #     print gray[10, :]

    gray = torgb(gray)

    #     print gray[10, :, 0]
    #     img = numpy_to_cv(rgb)

    # Initiate STAR detector
    #     star = cv2.FeatureDetector_create("STAR")  # @UndefinedVariable

    # Initiate BRIEF extractor

    #     brief = cv2.DescriptorExtractor_create("BRIEF")  # @UndefinedVariable

    sift = cv2.SIFT()

    # find the keypoints with STAR
    #     gray = cv2.cvtColor(rgb, cv2.COLOR_BGR2GRAY)

    kps = sift.detect(gray)  # , None, useProvidedKeypoints=False)

    #     print kps
    res = cv2.drawKeypoints(gray, kps)
    #     print res[10, :, 0]
    #
    #     for kp in kps:
    #         x = kp.x
    #         y = kp.y
    #         img[y, x, 0] = 255

    #     kp = star.detect(imgg, None)

    # compute the descriptors with BRIEF
    #     kp, des = brief.compute(rgb, kp)

    #     print kp
    #     print brief.getInt('bytes')
    #     print des

    return res


# Initiate STAR detector


@simple_block
@contract(rgb="array[HxWx3]", returns="array[HxWx3]")
def detect_and_plot_star(rgb, nfeatures=50):
    import cv2

    detector = cv2.FeatureDetector_create("STAR")

    r = detector.detect(rgb)  # , None, useProvidedKeypoints=False)
    print(("Number of STAR kps: %s" % len(r)))
    keypoints = r

    res = draw_keypoints(rgb, keypoints)
    return res


@simple_block
@contract(rgb="array[HxWx3]", returns="array[HxWx3]")
def detect_and_plot_dense(rgb, nfeatures=50):
    import cv2

    detector = cv2.FeatureDetector_create("Dense")

    r = detector.detect(rgb)  # , None, useProvidedKeypoints=False)
    print(("Number of dense kps: %s" % len(r)))
    keypoints = r

    res = draw_keypoints(rgb, keypoints)
    return res


@simple_block
@contract(rgb="array[HxWx3]", returns="array[HxWx3]")
def detect_and_plot_fast(rgb, nfeatures=50):
    import cv2

    detector = cv2.FeatureDetector_create("FAST")

    r = detector.detect(rgb)  # , None, useProvidedKeypoints=False)
    print(("Number of FAST kps: %s" % len(r)))
    keypoints = r

    res = draw_keypoints(rgb, keypoints)
    return res


def draw_keypoints(rgb, keypoints):
    import cv2

    try:
        raise ValueError()
        res = cv2.drawKeypoints(rgb, keypoints)
    except:
        res = rgb.copy()
        for kp in keypoints:
            x, y = kp.pt
            res[y, x, :] = 255

    return res


@simple_block
@contract(rgb="array[HxWx3]", returns="array[HxWx3]")
def detect_and_plot_orb(rgb, nfeatures=50):
    import cv2

    #     gray = torgb(gray)

    detector = cv2.ORB(nfeatures=nfeatures, edgeThreshold=0)  # @UndefinedVariable
    r = detector.detect(rgb)  # , None, useProvidedKeypoints=False)
    print(("Number of ORB kps: %s" % len(r)))
    keypoints = r

    #     print kps
    res = cv2.drawKeypoints(rgb, keypoints)
    #     print res[10, :, 0]
    #
    #     for kp in kps:
    #         x = kp.x
    #         y = kp.y
    #         img[y, x, 0] = 255

    #     kp = star.detect(imgg, None)

    # compute the descriptors with BRIEF
    #     kp, des = brief.compute(rgb, kp)

    #     print kp
    #     print brief.getInt('bytes')
    #     print des

    return res



#!/usr/bin/env python
from ..mp4conversion import convert_to_mp4
from optparse import OptionParser
from procgraph import logger
from procgraph.utils import wrap_script_entry_point


usage = """
    %cmd  [-o <output>] <filename>
"""


def tomp4(args):
    parser = OptionParser(usage=usage)

    parser.add_option("-o", dest="output", type="string", help="Output video.")

    (options, args) = parser.parse_args(args)

    if len(args) != 1:
        msg = "Exactly one argument necessary."
        raise Exception(msg)

    filename = args[0]
    return convert_to_mp4(filename, mp4=options.output, quiet=True)


def main():
    wrap_script_entry_point(tomp4, logger)


if __name__ == "__main__":
    main()



import os

from procgraph.utils import system_cmd_result, CmdException

from . import logger


CONTAINER_MP4 = "mp4"
CONTAINER_MOV = "mov"
CONTAINER_MKV = "mkv"
CONTAINER_AVI = "avi"
CONTAINERS = [CONTAINER_MOV, CONTAINER_MP4, CONTAINER_MKV, CONTAINER_AVI]


def supports_full_metadata(container):
    return container in [CONTAINER_MKV]


def guess_container(filename):
    _, ext = os.path.splitext(filename)
    ext = ext.lower()
    choices = {
        ".mp4": CONTAINER_MP4,
        ".mkv": CONTAINER_MKV,
        ".mov": CONTAINER_MOV,
        ".avi": CONTAINER_AVI,
    }
    if not ext in choices:
        raise ValueError(ext)
    return choices[ext]


def do_quickstart(source, target):
    # TODO: check file exists
    names = ["qtfaststart", "qt-faststart"]
    errors = []
    for name in names:
        cmd = [name, source, target]
        try:

            system_cmd_result(
                ".",
                cmd,
                display_stdout=True,
                display_stderr=True,
                raise_on_error=True,
                capture_keyboard_interrupt=False,
            )
            break
        except CmdException as e:
            errors.append(e)

    else:
        msg = "Could not call either of %s. " "The file will not be ready for streaming.\n%s" % (
            names,
            errors,
        )
        logger.error(msg)
        os.rename(source, target)



# A couple of functions for pretty errors
def aslist(x):
    if isinstance(x, dict):
        x = list(x.keys())
    if x:
        return ", ".join([e.__repr__() for e in sorted(x)])
    else:
        return "<empty>"


def raise_x_not_found(what, x, iterable, exception=ValueError):
    msg = x_not_found(what, x, iterable)
    raise exception(msg)


def x_not_found(what, x, iterable):
    """ Shortcut for creating pretty error messages. """
    # TODO: add guess in case of typos
    options = aslist(iterable)

    return "Could not find %s %r. I know the elements: %s." % (what, x, options)


def check_is_in(what, x, iterable, exception=ValueError):
    if not x in iterable:
        raise_x_not_found(what, x, iterable, exception)



# In the simplest case, you only need to import "Block" from procgraph.
from procgraph import Block


class BlockSimpleExample(Block):
    """ This is a documented example of the simplest block possible.
    
        This docstring will be included in the generated documentation.
    """

    # Define a name for the block using `Block.alias`.
    # If not given, the class name ('BlockExample') will be used.
    Block.alias("block_example")

    # Define the inputs for the blocks using `Block.input`.
    # It takes two parameters: name of the signal, and a docstring.
    # ProcGraph will generate wonderful documentation for you, and lets
    # you write documentation for what you are doing at almost every step:
    # take every chance to document things.
    Block.input("baz", "Measured baz in the particle accelerator.")

    # This block has one output as well.
    Block.output("baz_compensated", "Compensated baz value according to calibration.")

    # Now define the configuration for the block.
    # Pass the optional 'default' argument to define a default for the
    # parameter.
    # Without the default, the user *must* pass the configuration parameter.
    Block.config("bias", "Bias for the accelerator.", default=0)

    # Now let's write the logic for the block.

    # Very important! Do not use __init__ for initialization. Use init().
    # ProcGraph does a lot behind the scenes to let you have a streamlined
    # experiences, but sometimes there is a price to pay.
    #
    # def __init__(self):
    #     ...
    #
    # Initialize the block in the init() function.
    def init(self):
        # For this example, we have nothing to do here.
        pass

    # You will write the main logic in the update() function.
    # It is called whenever the input signals are updated.
    # In update() you do your computations, and update the value
    # of the output signals.
    def update(self):
        # You can access the value of the configuration parameters
        # using  `self.config.<parameter name>`
        bias = self.config.bias

        # You read the input signals using `self.input.<signal name>`:
        baz = self.input.baz

        baz_compensated = baz + bias

        # Finally, you set the output using `self.output.<signal name> = ...`.
        self.output.baz_compensated = baz_compensated
        # You're done: ProcGraph will propagate this value to the other blocks
        # connected...

        # Note that for stateless computations such as this, creating a Block
        # class is likely overkill -- use register_simple_block() with an
        # instantaneous function instead -- see next example.

    # Finish is called at the end of the execution.
    def finish(self):
        # For this example, we have nothing to do here.
        pass



__version__ = "1.1"

import sys
import traceback


class UserError(Exception):
    pass


def wrap_script_entry_point(function, logger, exceptions_no_traceback=(UserError,)):
    """
        Wraps the main() of a script.
        For Exception: we exit with value 2.
        
        :param exceptions_no_traceback: list of exceptions for which we 
         just print the error, and return 1.
        
    """
    try:
        ret = function(sys.argv[1:])
        if ret is None:
            ret = 0
        sys.exit(ret)
    except exceptions_no_traceback as e:
        logger.error(str(e))
        sys.exit(1)
    except Exception as e:
        logger.error(traceback.format_exc())
        sys.exit(2)
